@@ -4,11 +4,18 @@ import re
 import inspect
 
 class find_dialog:
+	def remove_tag(self):
+		if self.hit_end != -1 and self.hit_start != -1:
+			hit_start_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_start)
+			hit_end_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_end)
+			self.gui.textBuffer.remove_tag(self.found_tag, hit_start_iter, hit_end_iter)
+		
 	def destroy(self):
 		if self.gui.trace:
 			print inspect.getframeinfo(inspect.currentframe())[2]
 		self.xml.get_widget(self.name).destroy()
-
+		self.remove_tag()
+		
 	def saveSettings(self):
 		if self.gui.trace:
 			print inspect.getframeinfo(inspect.currentframe())[2]
@@ -119,12 +126,14 @@ class find_dialog:
 		"""
 		if self.gui.trace:
 			print inspect.getframeinfo(inspect.currentframe())[2]
+
+		self.remove_tag()
 		match_case_bool, global_bool, regex_bool, backwards_bool, find_text, replace_text = self._get_settings()
-		self.hit_start_iter = self.hit_end = None
+		self.hit_start = self.hit_end = -1
 		use_textBuffer = 1
 		current_tree_iter = self.gui.get_first_selected_iter()
 		
-		while self.hit_start_iter == None:
+		while self.hit_start == -1:
 			if not use_textBuffer:
 				if not global_bool:
 					return 0
@@ -133,15 +142,12 @@ class find_dialog:
 				if not current_tree_iter:
 					return 0
 				zone = self.gui.get_node_value(current_tree_iter)
-				start_offset = 0
-				end_offset = len(zone)
+				self.start_offset = 0
 			else:
 				start_mark = self.gui.textBuffer.get_insert()
 				start_iter = self.gui.textBuffer.get_iter_at_mark(start_mark)
 				end_iter = self.gui.textBuffer.get_end_iter()
-				start_offset = start_iter.get_offset()
-				end_offset = end_iter.get_offset()
-
+				self.start_offset = start_iter.get_offset()
 				zone = self.gui.textBuffer.get_text(start_iter, end_iter)
 			
 			if regex_bool:
@@ -157,8 +163,8 @@ class find_dialog:
 				
 				self.match = self.prog.search(zone)
 				if self.match:
-					hit_start = self.match.start()
-					hit_end = self.match.end()
+					self.hit_start = self.match.start()
+					self.hit_end = self.match.end()
 				else:
 					use_textBuffer = 0
 					continue
@@ -166,11 +172,11 @@ class find_dialog:
 				if not match_case_bool:
 					zone = zone.lower()
 					find_text = find_text.lower()
-				hit_start = zone.find(find_text)
-				if hit_start < 0:
+				self.hit_start = zone.find(find_text)
+				if self.hit_start < 0:
 					use_textBuffer = 0
 					continue
-				hit_end = hit_start + len(find_text)
+				self.hit_end = self.hit_start + len(find_text)
 
 			# OK - we got one, now return:
 			if not use_textBuffer:
@@ -179,10 +185,12 @@ class find_dialog:
 				self.gui.treeView.get_selection().unselect_all()
 				self.gui.treeView.get_selection().select_iter(current_tree_iter)
 				self.gui.treeView.scroll_to_cell(current_tree_path, None, 0, 0.5, 0.5)
-			self.hit_start_iter = self.gui.textBuffer.get_iter_at_offset(start_offset + hit_start)
-			self.hit_end_iter = self.gui.textBuffer.get_iter_at_offset(start_offset + hit_end)
-			self.gui.textBuffer.place_cursor(self.hit_end_iter)
-			self.gui.textBuffer.move_mark_by_name("selection_bound", self.hit_start_iter)
+			hit_start_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_start)
+			hit_end_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_end)
+			#self.gui.textBuffer.move_mark_by_name("selection_bound", hit_start_iter)
+			self.gui.textBuffer.apply_tag(self.found_tag, hit_start_iter,
+                hit_end_iter)	  
+			self.gui.textBuffer.place_cursor(hit_end_iter)
 			start_mark = self.gui.textBuffer.get_insert()
 			self.gui.textView.scroll_to_mark(start_mark, 0.0 , 1, 1.0, 0.5)
 			return 1
@@ -198,7 +206,11 @@ class find_dialog:
 			self.feedback(_("Not found"))
 		else:
 			self.feedback(_("Found"))
-			
+
+		w = self.xml.get_widget("findEntry")
+		if w:
+			w.grab_focus()
+
 	def on_findCloseButton_clicked(self, widget):
 		if self.gui.trace:
 			print inspect.getframeinfo(inspect.currentframe())[2]
@@ -215,25 +227,27 @@ class find_dialog:
 		if self.gui.trace:
 			print inspect.getframeinfo(inspect.currentframe())[2]
 		self.gui.sync_text_buffer()
-		if self.hit_start_iter == None:
+		if self.hit_start == -1:
 			return self._find_next()
 
 		match_case_bool, global_bool, regex_bool, backwards_bool, find_text, replace_text = self._get_settings()
 		
+		hit_start_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_start)
+		hit_end_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset + self.hit_end)
 		if regex_bool:
-			new_text = self.prog.sub(replace_text, self.gui.textBuffer.get_text(self.hit_start_iter, self.hit_end_iter))
+			new_text = self.prog.sub(replace_text, self.gui.textBuffer.get_text(hit_start_iter, hit_end_iter))
 		else:
 			new_text = replace_text
 		
-		self.gui.textBuffer.place_cursor(self.hit_end_iter)
-		self.gui.textBuffer.delete(self.hit_start_iter, self.hit_end_iter)
+		self.gui.textBuffer.place_cursor(hit_end_iter)
+		self.gui.textBuffer.delete(hit_start_iter, hit_end_iter)
 		self.gui.textBuffer.insert_at_cursor(new_text, len(new_text))
 		# "insert" mark is now at the _end_ of the replaced text.
 		# Now place the "selection" mark at the start of the replaced text:
 		start_mark = self.gui.textBuffer.get_insert()
 		start_iter = self.gui.textBuffer.get_iter_at_mark(start_mark)
-		start_offset = start_iter.get_offset()
-		new_selection_bound_iter = self.gui.textBuffer.get_iter_at_offset(start_offset - len(new_text))
+		self.start_offset = start_iter.get_offset()
+		new_selection_bound_iter = self.gui.textBuffer.get_iter_at_offset(self.start_offset - len(new_text))
 		self.gui.textBuffer.move_mark_by_name("selection_bound", new_selection_bound_iter)
 		#self.gui.dirty = 1 ... handled in on_textBuffer_changed()
 		
@@ -252,6 +266,10 @@ class find_dialog:
 			return
 
 		self.feedback(_("Replace?"))
+
+		w = self.xml.get_widget("findEntry")
+		if w:
+			w.grab_focus()
 		return
 		
 	def on_replaceAllButton_clicked(self, widget):
@@ -265,6 +283,10 @@ class find_dialog:
 			i = self._replace_next()
 			total = total + i
 		self.feedback(_("%d substitutions") % total)
+
+		w = self.xml.get_widget("findEntry")
+		if w:
+			w.grab_focus()
 		return
 	
 	def on_findCancelButton_clicked(self, widget):
@@ -279,6 +301,7 @@ class find_dialog:
 		w = self.xml.get_widget("findEntry")
 		if w:
 			w.set_text("")
+			w.grab_focus()
 
 		w = self.xml.get_widget("replaceEntry")
 		if w:
@@ -350,14 +373,17 @@ class find_dialog:
 		w = self.xml.get_widget("findEntry")
 		if w:
 			w.set_text(self.gui.client.get_string(self.gui.find_text_path))
+			w.select_region(0, -1)
 
 		w = self.xml.get_widget("replaceEntry")
 		if w:
 			w.set_text(self.gui.client.get_string(self.gui.replace_text_path))
-
+			
 		self.feedback("")
 		self._set_readonly(self.gui.readonly)
-		self.hit_start_iter = self.hit_end_iter = None
+		self.hit_start = self.hit_end = -1
+		self.found_tag = self.gui.textBuffer.create_tag(None,
+														background="lightblue")
 		
 # find_dialog
 
