@@ -100,7 +100,7 @@ class sort_dialog:
 		if w:
 			sort_ascending = w.get_active()
 		else:
-			print "_get_settings: Cant find sortAscendingRadioButton"
+			print _("_get_settings: Cant find sortAscendingRadioButton")
 		w = self.xml.get_widget("sortAlphabeticRadioButton")
 		if w:
 			sort_alpha = w.get_active()
@@ -136,7 +136,7 @@ class sort_dialog:
 			while not self.gui.same_iter(child, start_iter):
 				child = self.gui.treestore.iter_next(child)
 				if not child:
-					self.gui.msg("Internal error")
+					self.gui.msg(_("Internal error"))
 					return
 
 		if end_iter == 0:
@@ -218,7 +218,7 @@ class sort_dialog:
 				end_text = self.gui.textBuffer.get_end_iter()
 				body = self.gui.textBuffer.get_text(start_text, end_text)
 				if not body:
-					self.gui.msg("Nothing to sort!")
+					self.gui.msg(_("Nothing to sort!"))
 					return
 			text_list = body.split('\n')
 			if sort_alpha:
@@ -244,12 +244,12 @@ class sort_dialog:
 		"""
 		first_selected = self.gui.get_first_selected_iter()
 		if not first_selected:
-			self.gui.msg("Nothing selected")
+			self.gui.msg(_("Nothing selected"))
 			return
 
 		last_selected = self.gui.get_last_selected_iter()
 		if not last_selected:
-			self.gui.msg("Nothing selected")
+			self.gui.msg(_("Nothing selected"))
 			return
 
 		if self.gui.same_iter(first_selected, last_selected):
@@ -260,7 +260,7 @@ class sort_dialog:
 
  		parent = self.gui.treestore.iter_parent(first_selected)
 		if not parent:
-			self.gui.msg("No parent!")
+			self.gui.msg(_("No parent!"))
 			return
 
 		self._sort_item(parent, first_selected, last_selected, sort_ascending, sort_alpha, sort_sublevels, 0)
@@ -289,7 +289,7 @@ class sort_dialog:
 			"on_sortCancelButton_clicked":				self.on_sortCancelButton_clicked,
 		}
 		self.name = "sortDialog"
-		self.xml = gtk.glade.XML(self.gui.gui_filename, self.name)
+		self.xml = gtk.glade.XML(self.gui.gui_filename, self.name, domain="gjots2")
 		self.xml.signal_autoconnect(callbacks)
 
 		# Initialize the GUI state of the radio buttons and spin button
@@ -304,11 +304,11 @@ class sort_dialog:
 		if type == "tree":
 			self.type = "tree"
 			self.xml.get_widget("sortSublevelsSpinButton").set_sensitive(True)
-			self.xml.get_widget("sortDialog").set_title("Sort Tree")
+			self.xml.get_widget("sortDialog").set_title(_("Sort Tree"))
 		elif type == "text":
 			self.type = "text"
 			self.xml.get_widget("sortRangeFrame").hide()
-			self.xml.get_widget("sortDialog").set_title("Sort Text")
+			self.xml.get_widget("sortDialog").set_title(_("Sort Text"))
 
 		return
 		
