@@ -14,9 +14,9 @@
 prefix="/usr"
 
 rm -rf $prefix/lib/gjots2 $prefix/share/doc/gjots2-* $prefix/share/gjots2
-rm -f $prefix/bin/gjots2html $prefix/bin/docbook2gjots $prefix/bin/gjots2 $prefix/bin/gjots2docbook
+rm -f $prefix/bin/gjots2html $prefix/bin/docbook2gjots $prefix/bin/gjots2 $prefix/bin/gjots2docbook $prefix/bin/gjots2lpr
 rm -f $prefix/share/man/man1/gjots2html.1 $prefix/share/man/man1/docbook2gjots.1 $prefix/share/man/man1/gjots2.1 $prefix/share/man/man1/gjots2docbook.1
 rm -f $prefix/share/pixmaps/gjots.png $prefix/share/application/gjots2.desktop
-for LOCALE in en_US fr no nb ru it; do
+for LOCALE in en_US fr no nb ru cs it; do
 	rm -f $prefix/share/locale/$LOCALE/LC_MESSAGES/gjots2.mo
 done
\ No newline at end of file
